
#ifndef GALLNAV_H
#define GALLNAV_H
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: 
  Version: $ Rev: $

  2011-02-14 /JD: created

-*/

#include <iostream>
#include <map>
#include <vector>
#include <string>
#include <memory>

#include "gdata/gnav.h"
#include "gdata/geph.h"
#include "gdata/gdata.h"
#include "gdata/gnavgps.h"
#include "gdata/gnavglo.h"
#include "gdata/gnavgal.h"
#include "gdata/gnavqzs.h"
#include "gdata/gnavsbs.h"
#include "gdata/gnavbds.h"
#include "gutils/gconst.h"
#include "gutils/gtime.h"

#define MAX_GPS_PRN 32
#define MAX_GLO_PRN 24
#define MAX_GAL_PRN 30
#define NAV_BUF   1024


using namespace std;

namespace gnut {  

class t_gallnav : public t_gdata {

 public:
  t_gallnav();
  virtual ~t_gallnav();

  typedef map<t_gtime, shared_ptr<t_geph> > t_map_ref; // all data for a single satellite
  typedef map<string, t_map_ref>            t_map_sat; // all data for all satellites
   
//  typedef map<t_gtime,double>               t_map_sky_epo; // sat visibility of a single sat/site
//  typedef map<string, t_map_sky_epo>        t_map_sky_sat; // sat visibility of a single satellite

//  virtual int visibility( t_map_sky_sat& skysat, const t_gtriple& xyz,
//		          const t_gtime& beg = t_gtime::first_time(),
//		          const t_gtime& end = t_gtime::last_time() );
     
  virtual int pos( string sat, const t_gtime& t, double  xyz[3], double  var[3] = NULL, double  vel[3] = NULL ); // [m]
  virtual int clk( string sat, const t_gtime& t, double* clk,    double* var    = NULL, double* dclk   = NULL ); // [s]
  virtual void print(string sat, const t_gtime& t);
   
  virtual void clean_outer( const t_gtime& beg = t_gtime::first_time(), 
			    const t_gtime& end = t_gtime::last_time() );

  virtual t_gtime beg_gnav( string prn );                            // get first t_gnav epoch
  virtual t_gtime end_gnav( string prn );                            // get last  t_gnav epoch
  virtual t_gtime beg_time( string prn ){ return beg_gnav(prn); }    // get first t_gephrtcm epoch
  virtual t_gtime end_time( string prn ){ return end_gnav(prn); }    // get first t_gephrtcm epoch

  virtual set<string> satellites();                                  // get all satellites
  virtual unsigned int nepochs( const string& prn );                 // get number of epochs
  virtual int add( shared_ptr<t_gnav> nav );                         // add single navigation message
   
  virtual vector<shared_ptr<t_geph>> vec_nav( string prn = "" );     // get list of nav messages
  virtual vector<t_gtime>            vec_epo( string prn = "" );     // get list of epochs

  virtual void overwrite(bool b){ _overwrite = b; }                  // set/get overwrite mode
  virtual bool overwrite(){ return _overwrite; }

  virtual void com(bool b){ _com = b; }                              // position/clock reference point
  virtual bool com(){ return _com; }                                 // position/clock reference point
   
  virtual void offset( int i ){ _offset = i; }                       // set/get offset for satdata
  virtual int  offset(){ return _offset; }

 protected:   
//  virtual int _visibility( t_map_sky_sat& skysat, const t_gtriple& xyz,
//		   	   const t_gtime& beg = t_gtime::first_time(),
//		           const t_gtime& end = t_gtime::last_time() );

  virtual shared_ptr<t_geph> _find( string sat, const t_gtime& t );  // find appropriate t_geph element
  virtual shared_ptr<t_geph> _find( string sat, int iod          );  // find appropriate t_gnav element

  int                _com;         // position/clock reference point (com = true; apc = false);
  int                _offset;      // offset for RTCM corrections
  int                _nepoch;      // maximum number of epochs (0 = keep all)
  t_map_sat          _mapsat;      // map over all satellites (positions,..?)
  bool               _overwrite;   // rewrite/add only mode
  shared_ptr<t_geph> _null;        // null pointer
  
 private:
};

} // namespace

#endif
