
#ifndef GALLOBS_H
#define GALLOBS_H
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: observation container
  Version: $ Rev: $

  2012-05-02 /JD: created

-*/

#include <iostream>
#include <string.h>
#include <map>
#include <set>
#include <memory>

#include "gdata/gdata.h"
#include "gdata/gobsgnss.h"
#include "gdata/gsatgnss.h"
//#include "gdata/gsdbs.h"
#include "gdata/gsatdata.h"
#include "gutils/gsatview.h"
#include "gutils/gtypeconv.h"
#include "gutils/gconst.h"
#include "gutils/gtime.h"
#include "gutils/gnss.h"
#include "gutils/gsys.h"
#include "gutils/gobs.h"

#define DIFF_SEC 0.905  // [sec] returns observations within +- DIFF_SEC

using namespace std;

namespace gnut {  

typedef shared_ptr<t_gobsgnss>               t_spt_gobs;
   
class t_gallobs : public t_gdata {

 public:
  t_gallobs();
  virtual ~t_gallobs();

  typedef map<string, t_spt_gobs>              t_map_osat;      // all data-types/single epoch
  typedef map<t_gtime,t_map_osat>              t_map_oref;      // all data-types/all epochs/single object
  typedef map<string, t_map_oref>              t_map_oobj;      // all data-types/all epochs/all objects

  typedef map<string, pair<int,int>>           t_map_bsat;
  typedef map<t_gtime, t_map_bsat>             t_map_bepo;
  typedef map<GSYS,    t_map_bepo>             t_map_bsys;
   
  typedef map<string, pair<int,int>>           t_map_stt_sat;   // data statistics (have,total)
  typedef map<GOBS,   t_map_stt_sat>           t_map_stt_obs;
  typedef map<GSYS,   t_map_stt_obs>           t_map_stt_sys;
  typedef map<double, int>                     t_map_stt_smp;

  static  GOBSATTR select_attr(const t_map_stt_sys& stt,
			       GSYS gs, GOBSTYPE gt, GOBSBAND gb );                    // attr selection
   
  virtual set<string> stations();                                                      // get all station
  virtual t_map_stt_sys stat_obs(string site,                                          // get obs statistics
				 double* smp,                                          // set sampling (get if zero!)
                                 const t_gtime& beg = t_gtime::first_time(),           // req start period
			         const t_gtime& end = t_gtime::last_time(),            // req end period
		 	         t_map_sky_sat* sky_sat = 0,                           // based on skyview
		 	         t_map_stt_smp* smp_stt = 0,                           // get sample histogram
		 	         t_map_stt_sys* sky_stt = 0);                          // get obs statistics based on skyview
			                   
  virtual set<GSYS> sys(string site);
  virtual set<GSYS> sys(const string& site, const t_gtime& t);                         // get all systems for epoch
   
  virtual int gaps(string site, int g, map<t_gtime,int>& l, double& min, double& max); // gaps number
  virtual int block(string site, int b, int g, map<t_gtime,int>& l);                   // short block data
  virtual int nbands(string site, t_map_bsys& nb, int sampl = 0);                      // phase/code band number
  virtual int add_elevations(string site, t_gtriple xyz_rec, t_gallnav* gnav);         // add elevations for site

  virtual bool isSite(string site);
   
  virtual set<string> sats(const string& site, const t_gtime& t, GSYS gnss);           // get all satellites for epoch t and system

//  virtual t_gsdbs sdbs(const string& site, const t_gtime& t);                        // single differ. btw sats in epoch t

  virtual vector<t_gsatdata> obs(const string& site, const t_gtime& t);          // get all t_gsatdata for epoch t
  virtual vector<t_spt_gobs> obs_pt(const string& site, const t_gtime& t);       // get all t_gobsgnss pointers for epoch t
  virtual vector<t_spt_gobs> obs_prn_pt(const string& site, const string& prn, 
				        const t_gtime& beg, const t_gtime& end); // get all t_gobsgnss pointers for prn in interval
  virtual vector<t_gtime> epochs(const string& site);                            // get all t_gepochs for site
   
  virtual void clean_outer(const string& site = "",
                           const t_gtime& beg = t_gtime::first_time(),
			   const t_gtime& end = t_gtime::last_time());

  virtual t_gtime beg_obs(const string& site);           // get first t_gobs epoch for site
  virtual t_gtime end_obs(const string& site);           // get last  t_gobs epoch for site
   
  int addobs(t_spt_gobs obs);                            // add single station observation (P and L in meters !)
   
  void overwrite(bool b){ _overwrite = b; }              // set/get overwrite mode
  bool overwrite(){ return _overwrite; } 

  void maxepoch( unsigned int i ){ _nepoch = i; }        // set/get maximum number of epochs stored
  unsigned int maxepoch(){ return _nepoch; }

  unsigned int nepochs(const string& site);              // get number of epochs for station
  unsigned int nepochs(const string& site, const t_gtime& beg, const t_gtime& end,  const int& sampl, map<GSYS, pair<int,int> >& n);  // get number of epochs for station expect/have according to sampl

  virtual t_map_osat find( string site, const t_gtime& t); // find appropriate t_gobsgnss element for site/epoch
   
 protected:

  unsigned int      _nepoch;      // maximum number of epochs (0 = keep all)
  t_map_oobj        _mapobj;      // map over all objects (receivers)
  bool              _overwrite;   // rewrite/add only mode
  int               _find_epo(const string& site, const t_gtime& epo, t_gtime& tt);          // find epoch from the map w.r.t. DIFF_SEC   
    
 private:
};

} // namespace

#endif
