
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <cstring>

#include "gcoders/blq.h"
#include "gutils/gtriple.h"
#include "gutils/gtypeconv.h"
 
using namespace std;

namespace gnut {

/* ----------
 * constructor
 */
t_blq::t_blq( string version, int sz )
  : t_gcoder( version, sz )
{

}


/* ----------
 * BLQ header
 */
int t_blq::decode_head( char* buff, int sz, vector<string>& errmsg)
{ 
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _mutex.lock();

  if( t_gcoder::_add2buffer( buff, sz) == 0 ){ _mutex.unlock(); return 0; };

  string line;
  int consume = 0;
  int tmpsize = 0;
  while( ( tmpsize = t_gcoder::_getline( line )) >= 0 ){

    consume += tmpsize;
    
    if( line.find("END HEADER") != string::npos ){
      if( _log ) _log->comment(2,"blq","reading END OF HEADER ");

      t_gcoder::_consume(tmpsize);
      _mutex.unlock(); return -1;
    }
     
    t_gcoder::_consume(tmpsize);
  } // end while

  _mutex.unlock(); return consume;
}

   
/* ----------
 * BLQ body
 */
int t_blq::decode_data(char* buff, int sz, int& cnt, vector<string>& errmsg)
{
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _mutex.lock();
   
  if( t_gcoder::_add2buffer( buff, sz) == 0 ){ _mutex.unlock(); return 0; };

  string line;
  int consume = 0;
  int tmpsize = 0;
  bool complete = false; 
   
  while( ( tmpsize = t_gcoder::_getline( line )) >= 0 ){
    consume += tmpsize;        
    complete = false;            

    if (line.substr(0,2).compare("$$") == 0){
       unsigned int pos = 0;
       if ( (pos = line.find("lon/lat:")) != string::npos ){    // read lat, lon
	  line.erase(0,pos+8);
	  istringstream istr(line);
	  istr >> _lon >> _lat;
          t_gcoder::_consume(tmpsize);
	  continue;
       }else{
         t_gcoder::_consume(tmpsize);                 // delete comment
         continue;
       }       
    } else {
       istringstream istr(line);
       double a, b, c, d, e, f, g, h, i, j, k;
       istr >> a >> b >> c >> d >> e >> f >> g >> h >> i >> j >> k;
       if (! istr.fail()){
	  int ii = 1;
          _blqdata.ReSize(6,11);      
          _blqdata.Row(ii) << a << b << c << d << e << f << g << h << i << j << k;
	  while ( ii <= 6 )
	  {
	    if( ( tmpsize = t_gcoder::_getline( line ) ) < 0 ){
	      break;
  	    }
            istringstream istr(line);	     
            istr >> a >> b >> c >> d >> e >> f >> g >> h >> i >> j >> k;
//        cout << a << " " << b << " " << c << " " << d << " " << e << " " << f << " " << g << " " << h << " " << i << "  " << j << " " << k << endl;

            _blqdata.Row(ii) << a << b << c << d << e << f << g << h << i << j << k;	    
	    ii++; 
	    t_gcoder::_consume(tmpsize);
	    if (ii == 6) complete = true;
	  }
	  
       } else {	istringstream istr(line);	
	  istr >> _site;
	  t_gcoder::_consume(tmpsize);	  
       }              
    }

    if (complete) {              
      t_gotl otl;
      map<string,t_gdata*>::iterator it = _data.begin();
      if( it != _data.end() ) otl.setdata(_site, _lon, _lat, _blqdata);

      while( it != _data.end() ){
	if( it->second->id_type() == t_gdata::ALLOTL ){

          ((t_gallotl*)it->second)->add( otl );

	}
	it++;
      }
    }
     
#ifdef DEBUG     
    if (complete) {
      cout << "Site: " << _site << " lon: " << _lon << " lat: " << _lat << endl;
      cout << _blqdata << endl;
    } 
#endif	  

  }
   
  _mutex.unlock(); return -1;
}

} // namespace
