
#ifndef GIOF_H
#define GIOF_H

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: class derived from fstream
           - file name/mask added
           - support file hunting according to date/time
           - automated open/close handling
  Version: $ Rev: $

  2011-10-15 /JD: created

-*/

#include <fstream>
#include <string>

#ifdef BMUTEX
#include <boost/thread/mutex.hpp>
#endif

#include "gutils/gtime.h"
#include "gutils/gmutex.h"

using namespace std;

namespace gnut {

class t_giof : public fstream {

 public:
   t_giof( string mask = "" );
   virtual ~t_giof();

   int    mask( string mask );                     // set file mask
   string mask(){ return _mask; }                  // get file mask
   string name(){ return _name; }                  // get last filename
   
   int write( const char* buff, int size );        // writting
   int write( const string& s );

   int read( char* buff, int size );               // reading   
   void append(const bool& b=true );               // append mode [false/true]

   void tsys( t_gtime::t_tsys );                   // set/get time system for replacement
   t_gtime::t_tsys tsys();

 protected:
   string _replace();                              // replace mask to name 

   string             _mask;                       // original name mask
   string             _name;                       // actual (evaluated) name
   ios_base::openmode _omode;                      // output open mode
   bool               _repl;                       // replace if time-specific
   t_gtime::t_tsys    _tsys;                       // time system for replacement
   t_gmutex           _gmutex;

#ifdef BMUTEX
   boost::mutex       _mutex;                      // mutual exlusion
#endif
 private:
     
};

} // namespace

#endif
