
#ifndef GMODEL_H
#define GMODEL_H

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: base abstract class for models
  Version: $ Rev: $

  2014-11-27 /PV: created

-*/
#include <string>
#include <map>
#include <cmath>

#include "gio/glog.h"
#include "gall/gallpar.h"
#include "gutils/gtime.h"
#include "gdata/gsatdata.h"
#include "gset/gsetbase.h"
#include "gmodels/gtropo.h"

using namespace std;

namespace gnut {   

class t_gmodel
{
 public:
  virtual ~t_gmodel();

   void setSite(string site);

   void   glog(t_glog* l){ _log = l; }         
   
   virtual double cmpObs(t_gtime& epoch, t_gallpar& param, t_gsatdata&, bool, bool = false) = 0;
   
   virtual double windUp(t_gtime& epoch, const string, const ColumnVector&, const ColumnVector&){return 0.0;};

   virtual double tropoDelay(t_gtime& epoch, t_gallpar& param, t_gtriple ell, t_gsatdata& satdata){return 0.0;};

   virtual int outlierDetect( vector<t_gsatdata>& data, SymmetricMatrix& Qx, const SymmetricMatrix&, const ColumnVector&, vector<GOBSTYPE>& obstypes) = 0;
   
   t_gtropo* tropoModel;
   
 protected:
   t_gsetbase*         _settings;
   string              _site;
   t_glog*             _log;
   bool                _phase;
};

} // namespace

#endif //  GMODEL_H
