
#ifndef GTROPO_H
#define GTROPO_H 
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements troposphere model class
  Version: $ Rev: $

  2011-01-10 /JD: created

-*/

#include "gutils/gtime.h"
#include "gutils/gtriple.h"
#include "gall/gallsurf.h"
#include "gmodels/gnwmsurf.h"
#include "gmodels/ggpt.h"

using namespace std;

namespace gnut {   

class t_gtropo {
 public:
  t_gtropo();
  virtual ~t_gtropo();
   
  void nwm(t_gallsurf* nwm);      // set pointer to gsurf data

  virtual double getZHD(const t_gtriple&, const t_gtime&);
  virtual double getZWD(const t_gtriple&, const t_gtime&);
   
 protected:
  t_gpt        _gpt;
  t_gallsurf*  _nwm;

};


class t_saast : public t_gtropo {
 public:
   t_saast(){}
  ~t_saast(){}

   virtual double getSTD(const double&,    const double& );
   virtual double getZHD(const t_gtriple&, const t_gtime&);
   virtual double getZWD(const t_gtriple&, const t_gtime&);   
};


class t_davis : public t_gtropo {
 public:
   t_davis(){}
  ~t_davis(){}
   
   virtual double getZHD(const t_gtriple&, const t_gtime&);
   virtual double getZWD(const t_gtriple&, const t_gtime&);   
};


class t_hopf : public t_gtropo {
 public:
   t_hopf(){}
  ~t_hopf(){}
   
   virtual double getZHD(const t_gtriple&, const t_gtime&);
   virtual double getZWD(const t_gtriple&, const t_gtime&);
};


class t_baby : public t_gtropo {
 public:
   t_baby(){}
  ~t_baby(){}
   
   virtual double getZHD(const t_gtriple&, const t_gtime&);
   virtual double getZWD(const t_gtriple&, const t_gtime&);
};


class t_chao : public t_gtropo {
 public:
   t_chao(){}
  ~t_chao(){}
   
   virtual double getZHD(const t_gtriple&, const t_gtime&);
   virtual double getZWD(const t_gtriple&, const t_gtime&);
};


class t_ifad : public t_gtropo {
 public:
   t_ifad(){}
  ~t_ifad(){}

   virtual double getZHD(const t_gtriple&, const t_gtime&);
   virtual double getZWD(const t_gtriple&, const t_gtime&);
};


class t_askn : public t_gtropo {
 public:
   t_askn(){}
  ~t_askn(){}
   
   virtual double getZHD(const t_gtriple&, const t_gtime&);
   virtual double getZWD(const t_gtriple&, const t_gtime&);
   virtual double getZWD(const t_gtriple& Ell,
		         double  T,          // temperature [K]           
		         double  E,	     // water vapour [Pa]         
     		         double  dT,	     // T lapse rate [K/km]     
		         double  dE,	     // E lapse rate [-]        
		         double  dW,	     // ZWD lapse rate [-]        
		         double  ref,        // ZWD reference valeu [m]
                         double& ratio,      // combi ratio of E/ZWD lps rares [%]
			 double  tm=NWM_UNKNOWN
		        );
   virtual double getZWD_OLD(const t_gtriple& Ell,
		         double  T,          // temperature [K]           
		         double  E,	     // water vapour [Pa]         
     		         double  dT,	     // T lapse rate [K/km]     
		         double  dE,	     // E lapse rate [-]        
		         double  dW,	     // ZWD lapse rate [-]        
		         double  ref,        // ZWD reference valeu [m]
                         double& ratio,      // combi ratio of E/ZWD lps rares [%]
			 double  tm=NWM_UNKNOWN
		        );
};

} // namespace

#endif
