
#ifndef GPPP_H
#define GPPP_H

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements PPP client abstract class
  Version: $ Rev: $

  2012-06-06 /JD: created
  2013-06-18 /JD: joined gpppfilter & glsq

-*/
#ifdef BMUTEX
#include <boost/thread/mutex.hpp>
#endif

#include "gproc/gspp.h"
#include "gdata/gsatdata.h"
#include "gall/gallotl.h"
#include "gall/gallrslt.h"
#include "gall/gallsurf.h"
#include "gall/gallobj.h"
#include "gmodels/gtropo.h"
#include "gmodels/gpar.h"
#include "gmodels/gpar.h"
#include "gmodels/gtropo.h"
#include "gmodels/gtide.h"
#include "gmodels/gtide96.h"
#include "gmodels/gephplan.h"

namespace gnut {

class t_gppp : public virtual t_gspp
{
 public:
   t_gppp(string mark, t_gsetbase* set);
   virtual ~t_gppp();

   virtual void setOTL(t_gallotl*   gotl);
   virtual void setOBJ(t_gallobj*   gobj);
   virtual void setNWM(t_gallsurf*  gnwn);
   
   virtual bool isRunning();
    
 protected:   
   
   /** @brief Get settings from XML file and set local variables. */
   virtual int   _get_settings();
   
   /** @brief Print results. */
   virtual void  _prt_results(t_giof* giof, const t_gallrslt& rslt);

   int  _satNum(vector<t_gsatdata>& data, GSYS gs);
    
   t_gallobj*    _gobj;             /**< Objects transmitter/receiver. */
   t_gallotl*    _gotl;             /**< Ocean loading date. */
   t_gallsurf*   _gnwm;             /**< Objects for meteo 2D grid data. */
   t_gtide*      _tides;            /**< Tides. */

   bool          _running;          /**< Running status. */
   double        _sigAmbig;         /**< Ambiguity sigma (constraints). */
   
#ifdef BMUTEX
   boost::mutex  _mutex;
#endif
   t_gmutex      _gmutex;

 private:
};

} // namespace

#endif //GPPP_H
