
#ifndef GSPP_H
#define GSPP_H

#include "gutils/gtime.h"
#include "gutils/gmutex.h"
#include "gprod/gprodcrd.h"
#include "gprod/gprodtrp.h"
#include "gset/gsetbase.h"
#include "gset/gsetout.h"
#include "gset/gsetgen.h"
#include "gset/gsetgnss.h"
#include "gset/gsetrec.h"
#include "gset/gsetproc.h"
#include "gio/glog.h"
#include "gio/giof.h"
#include "gall/gallobs.h"
#include "gall/gallnav.h"
#include "gall/gallprod.h"
#include "gmodels/gbancroft.h"
#include "gmodels/gmodel.h"

namespace gnut {

class t_gspp
{
 public:
   t_gspp(string mark, t_gsetbase* set);
   virtual ~t_gspp();

// PROBLEM S POINTEREM!
//   void glog(t_glog* l){ _log = l; }
//   t_glog* glog()const{ return _log; }

   virtual int  processBatch(const t_gtime& beg, const t_gtime& end) = 0;
   
   virtual void setDAT(t_gallobs*   gobs, t_gallnav* gnav);   
           void setOUT(t_gallprod* products);
   
           void tropo(bool tropo);
           void phase(bool phase);
           void single_frq(bool single_frq); 
           void setgnss(GSYS sys);   
   string  site(){ return _site; }
   
 protected:
   /** @brief Get settings from XML file and set local variables. */
   virtual int   _get_settings();   
   
   /** @brief Set output file (empty). */
   virtual void  _setOut();
   
   /** @brief Set log file. */
   virtual void  _setLog();      
   
   bool          _valid_crd_xml();
   bool          _valid_ztd_xml();
   
   t_gmodel*     _gModel;           /**< models. */  
   string        _site;             /**< Site internal ID. */
   t_gsetbase*   _set;              /**< Base setting. */
   t_glog*       _log;              /**< Log output. */
   t_giof*       _res;              /**< Residuals output. */
   t_gallobs*    _gobs;             /**< Observation data. */
   t_gallnav*    _gnav;             /**< Objects for ephemerides. */
   t_gallprod*   _allprod;          /**< Products output. */
   bool          _phase;            /**< Phase is used. */
   bool          _single_frq;       /**< single frequency. */
   bool          _tropo;            /**< Tropo is estimated. */
   GSYS          _gnss;             /**< GNSS system to be used. */
   string        _mf;               /**< Mapping function. */
   double        _minElev;          /**< Elevation cut-off. */
   int           _sampling;         /**< Sampling interval. */
   bool          _initialized;      /**< Initialized status. */
   set<string>   _sats;             /**< Satellites to be used. */
   string        _crd_constr;       /**< Constraining coordinates. */
   t_gtriple     _aprox_crd_xml;    /**< Approximate coordinates. */
   double        _aprox_ztd_xml;    /**< Approximate ztd. */
   
   // init sigma
   double        _sig_init_crd;     /**< Initial coordinates sigma. */
   double        _sig_init_ztd;     /**< Initial ZTD sigma. */   
   
   // ISB sigma
   double        _sig_init_glo;     /**< Initial GLONASS ISB sigma. */
   double        _sig_init_gal;     /**< Initial Galileo ISB sigma. */
   double        _sig_init_bds;     /**< Initial BeiDou ISB sigma. */
   double        _sig_init_qzs;     /**< Initial QZSS ISB sigma. */
   
   // observations sigma

   double        _sigCodeGPS;       /**< Code sigma.  - GPS*/
   double        _sigPhaseGPS;      /**< Phase sigma. - GPS*/
   double        _sigCodeGLO;       /**< Code sigma.  - GLONASS*/
   double        _sigPhaseGLO;      /**< Phase sigma. - GLONASS*/
   double        _sigCodeGAL;       /**< Code sigma.  - Galileo*/
   double        _sigPhaseGAL;      /**< Phase sigma. - Galileo*/   
   double        _sigCodeBDS;       /**< Code sigma.  - BeiDou*/
   double        _sigPhaseBDS;      /**< Phase sigma. - BeiDou*/
   double        _sigCodeQZS;       /**< Code sigma.  - QZSS*/
   double        _sigPhaseQZS;      /**< Phase sigma. - QZSS*/   
   

};

} // namespace

#endif

