
#ifndef GSPPFLT_H
#define GSPPFLT_H

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements spp client
  Version: $ Rev: $

  2014-11-24 /PV: created

-*/

#include "gproc/gspp.h"
#include "gmodels/gdop.h"
#include "gproc/gflt.h"
#include "gall/gallpar.h"
#include "gutils/gsysconv.h"
#include "gmodels/gstochasticmodel.h"
#include "gset/gsetflt.h"

namespace gnut {

class t_gsppflt : public virtual t_gspp
{
 public:
   t_gsppflt();      
   t_gsppflt(string mark, t_gsetbase* set);
  ~t_gsppflt();
   
   virtual int processBatch(const t_gtime& beg, const t_gtime& end);
   
 protected:
   virtual void   _predict();
   virtual int    _satPos(t_gtime&, t_gsatdata&);
   virtual int    _prepareData();
   virtual int    _apply_tides(t_gtime& _epoch, t_gtriple& xRec);
   virtual int    _processEpoch(const t_gtime& runEpoch);      
           int   _addObsP(t_gsatdata& satdata, unsigned int& iobs, t_gtriple& ell, Matrix& A, ColumnVector& l, DiagonalMatrix& P);
   virtual int   _addObsL(t_gsatdata& satdata, unsigned int& iobs, t_gtriple& ell, Matrix& A, ColumnVector& l, DiagonalMatrix& P);
   
         double   _weightObs(t_gsatdata& satdata);

   virtual void   _timeUpdate(const t_gtime& epo);
           void   _syncSys();
   
   vector<t_gsatdata> _data;

// Models   
   t_randomwalk*       _trpStoModel;
   t_whitenoise*       _gloStoModel;
   t_whitenoise*       _galStoModel;
   t_whitenoise*       _bdsStoModel;
   t_whitenoise*       _qzsStoModel;
   t_whitenoise*       _clkStoModel;
   t_whitenoise*       _crdStoModel;    
   
   
// Parameters and covariance matrix   
   t_gallpar           _param;
   SymmetricMatrix     _Qx; 

// Time for constraining ZTD
   t_gtime           _cEpo_beg; 
   t_gtime           _cEpo_end;   
   
// Noise matrix
   DiagonalMatrix      _Noise;
   
// Epoch time
   t_gtime           _epoch;

// Estimation objects   
   t_gflt*           _filter;
   
   int               _numSat(GSYS gsys);   
   ColumnVector      _vBanc; 
   t_gdop            _dop; 
   int               _cntrep;      
   bool              _smooth;        
   
};

} // namespace

#endif // GSPPFLT_H
