
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include "gprod/gprodtrp.h"

using namespace std;

namespace gnut {
   
// constructor
// ----------
t_gprodtrp::t_gprodtrp(const t_gtime& t, shared_ptr<t_gobj> pt)
 : t_gprod(t, pt),
  _ztd(0.0),
  _grd_N(0.0),
  _grd_E(0.0),
  _ztd_rms(0.0),
  _grd_N_rms(0.0),
  _grd_E_rms(0.0)
{ 
   gtrace("t_gprodtrp::t_gprodtrp");
   
   id_type(TRP);
   id_group(GRP_PRODUCT);   
}

// destructor
// --------------
t_gprodtrp::~t_gprodtrp()
{
}


// add ztd
// --------------------------
void t_gprodtrp::ztd(const double& ztd, const double& rms)
{
  _ztd     = ztd;
  _ztd_rms = rms;
}

// get ztd
// --------------------------
double t_gprodtrp::ztd() const
{
  return _ztd;
}

// get ztd rms
// -------------------------
double t_gprodtrp::ztd_rms() const
{
  return _ztd_rms;
}


// add grd
// --------------------------
void t_gprodtrp::grd_N(const double& grd, const double& rms)
{
  _grd_N     = grd;
  _grd_N_rms = rms;
}

// get grd
// --------------------------
double t_gprodtrp::grd_N() const
{
  return _grd_N;
}

// get grd rms
// -------------------------
double t_gprodtrp::grd_N_rms() const
{
  return _grd_N_rms;
}

// add grd
// --------------------------
void t_gprodtrp::grd(const double& grdN, const double& grdE,
		     const double& rmsN, const double& rmsE )
{
  _grd_N     = grdN;
  _grd_N_rms = rmsN;
  _grd_E     = grdE;
  _grd_E_rms = rmsE;
}


// add grd
// --------------------------
void t_gprodtrp::grd_E(const double& grd, const double& rms)
{
  _grd_E     = grd;
  _grd_E_rms = rms;
}

// get grd
// --------------------------
double t_gprodtrp::grd_E() const
{
   return _grd_E;
}

// get grd rms
// -------------------------
double t_gprodtrp::grd_E_rms() const
{
   return _grd_E_rms;
}

} // namespace
