
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <iomanip>
#include <sstream>

#include "gset/gsetflt.h"

using namespace std;
using namespace pugi;

namespace gnut {

// Constructor
// ----------
t_gsetflt::t_gsetflt() 
 : t_gsetbase()
{
   _set.insert(XMLKEY_FLT);
  _noise_clk      = 1000.0;
  _noise_glo      = 1000.0;
  _noise_gal      = 1000.0;
  _noise_bds      = 1000.0;
  _noise_qzs      = 1000.0;   
  _noise_crd      = 100.0;
  _rndwk_ztd      = 3;
  _rndwk_grd      = 0.3;   
  _method_flt     = "SRCF";
  _method_smt     = "RTSSVD";
  _reset_amb      = 0;
  _reset_par      = 0;
  _delay          = 0;
}


// Destructor
// ----------
t_gsetflt::~t_gsetflt()
{}


// Return value
// ----------
string t_gsetflt::method_flt()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif
   _gmutex.lock();
   
   string tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("method_flt").value();
   _gmutex.unlock(); return tmp;
}

// Return value
// ----------
string t_gsetflt::method_smt()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif
   _gmutex.lock();
   
   string tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("method_smt").value();
   _gmutex.unlock(); return tmp;
}


// Return value
// ----------
double t_gsetflt::noise_clk()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock();
   
   double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("noise_clk").as_double();
   _gmutex.unlock(); return tmp;
}


// Return value
// ----------
double t_gsetflt::noise_crd()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock();
   
   double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("noise_crd").as_double();
   
   _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetflt::noise_glo()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock();
   
   double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("noise_glo").as_double();

   _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetflt::noise_gal()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock();
   
   double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("noise_gal").as_double();
   _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetflt::noise_bds()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock();
   
   double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("noise_bds").as_double();
   _gmutex.unlock(); return tmp;
}   
   
// Return value
// ----------
double t_gsetflt::noise_qzs()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock();
   
   double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("noise_qzs").as_double();
   _gmutex.unlock(); return tmp;
}   
   
// Return value
// ----------
double t_gsetflt::rndwk_ztd()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock(); 
   
   double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("rndwk_ztd").as_double();

   _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetflt::rndwk_grd()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock(); 
   
   double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("rndwk_grd").as_double();
   _gmutex.unlock(); return tmp;
}

// Return value
// ----------
int t_gsetflt::reset_amb()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock(); 
   
   int tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("reset_amb").as_int();
   _gmutex.unlock(); return tmp;
}

// Return value
// ----------
int t_gsetflt::reset_par()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock(); 
   
   int tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("reset_par").as_int();
   _gmutex.unlock(); return tmp;
}

// Return value
// ----------
int t_gsetflt::smt_delay()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
   _gmutex.lock(); 
   
   int tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT).attribute("smt_delay").as_int();
   _gmutex.unlock(); return tmp;
}

// settings check
// ----------
void t_gsetflt::check()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();

  // check existence of nodes/attributes
  xml_node parent = _doc.child(XMLKEY_ROOT);
  xml_node node = _default_node(parent, XMLKEY_FLT);

  // check existence of attributes
  _default_attr(node,"method_flt",_method_flt);
  _default_attr(node,"method_smt",_method_smt);   
  _default_attr(node,"noise_clk", _noise_clk );
  _default_attr(node,"noise_crd", _noise_crd );
  _default_attr(node,"noise_glo", _noise_glo );
  _default_attr(node,"noise_gal", _noise_gal );   
  _default_attr(node,"rndwk_ztd", _rndwk_ztd );
  _default_attr(node,"rndwk_grd", _rndwk_grd );   
  _default_attr(node,"reset_amb", _reset_amb );
  _default_attr(node,"reset_par", _reset_par );
  _default_attr(node,"smt_delay", _delay );   

#ifdef BMUTEX   
  lock.unlock();
#endif   
 
   _gmutex.unlock(); return;
}


// help body
// ----------
void t_gsetflt::help()
{
#ifdef BMUTEX   
   boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();

  cerr << " <filter \n"
       << "   method_flt=\""     <<  _method_flt  << "\" \n"
       << "   method_smt=\""     <<  _method_smt  << "\" \n"     
       << "   noise_clk=\""      <<  _noise_clk   << "\" \n"
       << "   noise_crd=\""      <<  _noise_crd   << "\" \n"
       << "   rndwk_ztd=\""      <<  _rndwk_ztd   << "\" \n"
       << "   rndwk_grd=\""      <<  _rndwk_grd   << "\" \n"
       << "   smt_delay=\""      <<  _delay   << "\" \n"
       << "  />\n";

  cerr << "\t<!-- filter description:\n"
       << "\t method_flt    .. type of filtering method (SRCF, Kalman)\n"
       << "\t method_smt    .. type of smoothing method (RTS)\n"     
       << "\t noise_clk     .. white noise for clocks \n"
       << "\t noise_crd     .. white noise for coordinates \n"
       << "\t rndwk_ztd     .. random walk process for ZTD [mm/sqrt(hour)] \n"
       << "\t -->\n\n";

#ifdef BMUTEX   
  lock.unlock();
#endif
   
   _gmutex.unlock(); return;   
}

} // namespace
