
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <iostream>
#include <iomanip>
#include <sstream>
#include <algorithm>

#include "gset/gsetnwm.h"
#include "gall/gallsurf.h"

using namespace std;
using namespace pugi;

namespace gnut {

// Constructor
// ----------
t_gsetnwm::t_gsetnwm() 
 : t_gsetbase()
{
  _set.insert(XMLKEY_NWM);
  _min_lat  = MIN_LAT;
  _max_lat  = MAX_LAT;
  _min_lon  = MIN_LON;
  _max_lon  = MAX_LON;
  _inc_fac  =     1.0;
  _negative =       0;
  _interp_space = "hor2ver";
  _interp_time  = "linear";
  _interp_vert  = "reduce";  // integrate
}


// Destructor
// ----------
t_gsetnwm::~t_gsetnwm()
{}


// Return set
// ----------
set<string> t_gsetnwm::param()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  set<string> tmp = t_gsetbase::_setval( XMLKEY_NWM, "param" );
  _gmutex.unlock(); return tmp;
}


// Return set
// ----------
set<string> t_gsetnwm::point()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  set<string> tmp = t_gsetbase::_setval( XMLKEY_NWM, "point" );
  _gmutex.unlock(); return tmp;
}


// Return set
// ----------
set<string> t_gsetnwm::surface()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  set<string> tmp = t_gsetbase::_setval( XMLKEY_NWM, "surface" );
  _gmutex.unlock(); return tmp;
}


// Return set
// ----------
set<string> t_gsetnwm::profile()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  set<string> tmp = t_gsetbase::_setval( XMLKEY_NWM, "profile" );
  _gmutex.unlock(); return tmp;
}

// Return set
// ----------
set<string> t_gsetnwm::assess()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  set<string> tmp = t_gsetbase::_setval( XMLKEY_NWM, "assess" );
  _gmutex.unlock(); return tmp;
}


// Return set
// ----------
set<string> t_gsetnwm::clean()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  set<string> tmp = t_gsetbase::_setval( XMLKEY_NWM, "clean" );
  _gmutex.unlock(); return tmp;
}

/*
// Get level request
// ----------
set<string> t_gsetnwm::surface()
{
  set<string> tmp;
  string str;
  string id = "level";
  string ID = id; transform(ID.begin(), ID.end(), ID.begin(), ::toupper);

  for( xml_node node = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).first_child(); node; node = node.next_sibling() ){
    if( node.name() == id || node.name() == ID ){
      istringstream is( node.child_value() );
      while( is >> str && !is.fail() ){
	transform(str.begin(), str.end(), str.begin(), ::toupper);
	tmp.insert( str );
//	cout << "LEVEL = |" << str << "|\n";
      }
    }
  }
  return tmp;
}


// Get level request
// ----------
set<string> t_gsetnwm::param()
{
  set<string> tmp;
  string str;
  string id = "param";
  string ID = id; transform(ID.begin(), ID.end(), ID.begin(), ::toupper);

  for( xml_node node = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).first_child(); node; node = node.next_sibling() ){
    if( node.name() == id || node.name() == ID ){
      istringstream is( node.child_value() );
      while( is >> str && !is.fail() ){
	transform(str.begin(), str.end(), str.begin(), ::toupper);
	tmp.insert( str );
//	cout << "SURFACE = |" << str << "|\n";
      }
    }
  }
  return tmp;
}
*/

// Return value
// ----------
double t_gsetnwm::inc_fac()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("inc_fac").as_double();

  _gmutex.unlock(); return tmp;
}
   
// Return value
// ----------
double t_gsetnwm::min_rad()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("min_rad").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetnwm::min_lat()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("min_lat").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetnwm::max_lat()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("max_lat").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetnwm::min_lon()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("min_lon").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetnwm::max_lon()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("max_lon").as_double();

  _gmutex.unlock(); return tmp;
}


// Return value
// ----------
int t_gsetnwm::negative()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("negative").as_int();

  _gmutex.unlock(); return tmp;
}


// Return value
// ----------
string t_gsetnwm::interp_space()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
     
  string tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("interp_space").as_string();
   
  transform(tmp.begin(), tmp.end(), tmp.begin(), ::toupper);
   
  _gmutex.unlock(); return tmp;
}


// Return value
// ----------
string t_gsetnwm::interp_time()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  string tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("interp_time").as_string();

  transform(tmp.begin(), tmp.end(), tmp.begin(), ::toupper);

  _gmutex.unlock(); return tmp;
}


// Return value
// ----------
string t_gsetnwm::interp_vert()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
     
  string tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_NWM).attribute("interp_vert").as_string();
   
  transform(tmp.begin(), tmp.end(), tmp.begin(), ::toupper);
   
  _gmutex.unlock(); return tmp;
}


// settings check
// ----------
void t_gsetnwm::check()
{
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
     
  // check existence of nodes/attributes
  xml_node parent = _doc.child(XMLKEY_ROOT);
  xml_node node = _default_node(parent, XMLKEY_NWM);

  // check existence of attributes
  _default_attr(node,"inc_fac",      _inc_fac);
  _default_attr(node,"min_rad",      _min_rad);   
  _default_attr(node,"min_lat",      _min_lat);
  _default_attr(node,"max_lat",      _max_lat);
  _default_attr(node,"min_lon",      _min_lon);
  _default_attr(node,"max_lon",      _max_lon);
  _default_attr(node,"negative",     _negative);
  _default_attr(node,"interp_space", _interp_space);
  _default_attr(node,"interp_time",  _interp_time);
  _default_attr(node,"interp_vert",  _interp_vert);

#ifdef BMUTEX   
  lock.unlock();
#endif
  
   _gmutex.unlock(); return;
}


// help body
// ----------
void t_gsetnwm::help()
{
#ifdef BMUTEX
  boost::mutex::scoped_lock lock(_mutex);
#endif
   _gmutex.lock();
   
  cerr << " <nwm"
       << "   inc_fac=\""      <<  _inc_fac       << "\""
       << "   min_rad=\""      <<  _min_rad       << "\""     
       << "   min_lat=\""      <<  _min_lat       << "\""
       << "   max_lat=\""      <<  _max_lat       << "\""
       << "   min_lon=\""      <<  _min_lon       << "\""
       << "   max_lon=\""      <<  _max_lon       << "\""
       << "   negative=\""     <<  _negative      << "\""
       << "   interp_space\""  <<  _interp_space  << "\""
       << "   interp_time\""   <<  _interp_time   << "\""
       << "   interp_vert\""   <<  _interp_vert   << "\""
       << " />\n";

  cerr << "   <param> ZTD ZHD ZWD IWV PRES TEMP TM </param>  \t\t <!-- output parameters -->\n"
       << "   <level> surf geoid elips levels      </level>  \t\t <!-- output levels -->\n"
       << " </nwm>";

  cerr << "\t<!-- NWM procesing description:\n"
       << "\t inc_fac        double  .. increment factor (multiplicator for dLat/dLon around point)\n"
/*ME*/ << "\t min_rad        double  .. radius factor (select nearest grid point)\n"     
       << "\t min_lat        double  .. minimum latitude (region)\n"
       << "\t max_lat        double  .. maximum latitude (region)\n"
       << "\t min_lon        double  .. minimum longitude (region)\n"
       << "\t max_lon        double  .. maximum longitude (region)\n" 
       << "\t negative          int  .. allow negative lps-rate/exp-decay\n"
       << "\t interp_space   string  .. ver2hor hor2ver\n"
       << "\t interp_time    string  .. linear spline\n"
       << "\t interp_vert    string  .. integrate reduce\n"
       << "\t -->\n\n";
   

#ifdef BMUTEX   
  lock.unlock();
#endif   
   
   _gmutex.unlock(); return;   
}

} // namespace
