
#ifndef GSETNWM_H
#define GSETNWM_H

#define XMLKEY_NWM "nwm"

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements data extraction setting
  Version: $ Rev: $

  2012-10-23 /JD: created

-*/

#include <string>
#include <iostream>

#ifdef BMUTEX
  #include <boost/thread/mutex.hpp>
#endif

#include "gio/glog.h"
#include "gutils/gtypeconv.h"
#include "gmodels/gnwmsurf.h"
#include "gset/gsetbase.h"

using namespace std;
using namespace pugi;

namespace gnut {

class t_gsetnwm : public virtual t_gsetbase
{
 public:
   t_gsetnwm();
  ~t_gsetnwm();
  
   void check();                                 // settings check
   void help();                                  // settings help

   double inc_fac();
   double min_rad();
   double min_lat();
   double max_lat();
   double min_lon();
   double max_lon();
   int   negative();
   string interp_space();
   string interp_time();
   string interp_vert();

   set<string> param();                          // get param request
   set<string> point();                          // get point request
   set<string> surface();                        // get surface request
   set<string> profile();                        // get profile request
   set<string> assess();                         // get assess request
   set<string> clean();                          // get clean request

 protected:

   set<string> _param;
   set<string> _point;
   set<string> _surface;
   set<string> _profile;
   set<string> _assess;
   double _inc_fac;
   double _min_rad;
   double _min_lat;
   double _max_lat;
   double _min_lon;
   double _max_lon;
   int    _negative;
   string _interp_space;
   string _interp_time;
   string _interp_vert;

 private:
};

} // namespace

#endif
