
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <iostream>
#include <iomanip>
#include <sstream>

#include "gset/gsetproc.h"

using namespace std;
using namespace pugi;

namespace gnut {

// Constructor
// ----------
t_gsetproc::t_gsetproc() 
 : t_gsetbase()
{
   _set.insert(XMLKEY_PROC);
  _phase          = true;
  _tropo          = true;
  _gradient       = false;
  _single_frq     = false;
  _tropo_model    = "saastamoinen";
  _tropo_mf       = "gmf";
  _grad_mf       = "tilt";   
  _sigma_C_GPS    = 2;
  _sigma_C_GLO    = 4;
  _sigma_C_GAL    = 3;
  _sigma_C_BDS    = 5;
  _sigma_C_QZS    = 5;   
  _sigma_L_GPS    = 0.02;
  _sigma_L_GLO    = 0.03;
  _sigma_L_GAL    = 0.03;
  _sigma_L_BDS    = 0.04;
  _sigma_L_QZS    = 0.03;   
  _sig_init_ztd   = 0.1;
  _sig_init_amb   = 1000.0;
  _sig_init_crd   = 100.0;
  _sig_init_glo   = 1000.0;
  _sig_init_gal   = 1000.0;   
  _minimum_elev   = 10;
}


// Destructor
// ----------
t_gsetproc::~t_gsetproc()
{}


// Return value
// ----------
bool   t_gsetproc::tropo()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  bool tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("tropo").as_bool();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
bool   t_gsetproc::gradient()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  bool tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("gradient").as_bool();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
bool   t_gsetproc::phase()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  bool tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("phase").as_bool();

  _gmutex.unlock(); return tmp;
}
   
// Return value
// ----------
bool   t_gsetproc::single_frq()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  bool tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("single_frq").as_bool();

  _gmutex.unlock(); return tmp;
}   

// Return value
// ----------
double t_gsetproc::sigma_C_GPS()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
  
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_C_GPS").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetproc::sigma_C_GLO()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_C_GLO").as_double();

  _gmutex.unlock(); return tmp;
}
   
// Return value
// ----------
double t_gsetproc::sigma_C_GAL()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_C_GAL").as_double();

  _gmutex.unlock(); return tmp;
}   

// Return value
// ----------
double t_gsetproc::sigma_C_BDS()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_C_BDS").as_double();

  _gmutex.unlock(); return tmp;
}   

// Return value
// ----------
double t_gsetproc::sigma_C_QZS()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_C_QZS").as_double();

  _gmutex.unlock(); return tmp;
}   
   
// Return value
// ----------
double t_gsetproc::sigma_L_GPS()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_L_GPS").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetproc::sigma_L_GLO()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_L_GLO").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetproc::sigma_L_GAL()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_L_GAL").as_double();

  _gmutex.unlock(); return tmp;
}   

// Return value
// ----------
double t_gsetproc::sigma_L_BDS()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_L_BDS").as_double();

  _gmutex.unlock(); return tmp;
}   
   
// Return value
// ----------
double t_gsetproc::sigma_L_QZS()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sigma_L_QZS").as_double();

  _gmutex.unlock(); return tmp;
}   
   
// Return value
// ----------
double t_gsetproc::sig_init_ztd()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  
   _gmutex.unlock(); return _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sig_init_ztd").as_double();
}


// Return value
// ----------
double t_gsetproc::sig_init_crd()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sig_init_crd").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetproc::sig_init_amb()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sig_init_amb").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetproc::sig_init_glo()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sig_init_glo").as_double();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
double t_gsetproc::sig_init_gal()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sig_init_gal").as_double();

  _gmutex.unlock(); return tmp;
}
   
// Return value
// ----------
double t_gsetproc::sig_init_bds()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sig_init_bds").as_double();

  _gmutex.unlock(); return tmp;
}   

// Return value
// ----------
double t_gsetproc::sig_init_qzs()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("sig_init_qzs").as_double();

  _gmutex.unlock(); return tmp;
}   
   
// Return value
// ----------
double t_gsetproc::minimum_elev()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif
  _gmutex.lock();
   
  double tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("minimum_elev").as_double();

  _gmutex.unlock(); return tmp;
}


// Return value
// ----------
string t_gsetproc::tropo_model()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  string tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("tropo_model").value();

  _gmutex.unlock(); return tmp;
}


// Return value
// ----------
string t_gsetproc::tropo_mf()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  string tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("tropo_mf").value();

  _gmutex.unlock(); return tmp;
}

// Return value
// ----------
string t_gsetproc::grad_mf()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  string tmp = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC).attribute("grad_mf").value();

  _gmutex.unlock(); return tmp;
}

// settings check
// ----------
void t_gsetproc::check()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  // check existence of nodes/attributes
  xml_node parent = _doc.child(XMLKEY_ROOT);
  xml_node node = _default_node(parent, XMLKEY_PROC);

  // check existence of attributes
  _default_attr(node,"phase", _phase);
  _default_attr(node,"tropo", _tropo);
  _default_attr(node,"gradient", _gradient);
  _default_attr(node,"single_frq", _single_frq);   
  _default_attr(node,"tropo_model", _tropo_model);
  _default_attr(node,"tropo_mf", _tropo_mf);
  _default_attr(node,"grad_mf", _grad_mf);   
  _default_attr(node,"sigma_C_GPS", _sigma_C_GPS);
  _default_attr(node,"sigma_L_GPS", _sigma_L_GPS);
  _default_attr(node,"sigma_C_GLO", _sigma_C_GLO);
  _default_attr(node,"sigma_L_GLO", _sigma_L_GLO);
  _default_attr(node,"sigma_C_GAL", _sigma_C_GAL);
  _default_attr(node,"sigma_L_GAL", _sigma_L_GAL);
  _default_attr(node,"sigma_C_BDS", _sigma_C_BDS);
  _default_attr(node,"sigma_L_BDS", _sigma_L_BDS);
  _default_attr(node,"sigma_C_QZS", _sigma_C_QZS);
  _default_attr(node,"sigma_L_QZS", _sigma_L_QZS);
  _default_attr(node,"sig_init_crd", _sig_init_crd);
  _default_attr(node,"sig_init_ztd", _sig_init_ztd);
  _default_attr(node,"sig_init_amb", _sig_init_amb);
  _default_attr(node,"sig_init_glo", _sig_init_glo);
  _default_attr(node,"sig_init_gal", _sig_init_gal);
  _default_attr(node,"sig_init_bds", _sig_init_bds);
  _default_attr(node,"sig_init_qzs", _sig_init_qzs);   
  _default_attr(node,"minimum_elev", _minimum_elev);
  
#ifdef BMUTEX   
  lock.unlock();
#endif
   
  _gmutex.unlock(); return; 
}


// help body
// ----------
void t_gsetproc::help()
{
#ifdef BMUTEX   
  boost::mutex::scoped_lock lock(_mutex);
#endif   
  _gmutex.lock();
   
  cerr << " <process \n"
       << "   phase=\""          <<  _phase          << "\" \n"
       << "   tropo=\""          <<  _tropo          << "\" \n"
       << "   gradient=\""       <<  _gradient       << "\" \n"     
       << "   tropo_model=\""    <<  _tropo_model    << "\" \n"
       << "   tropo_mf=\""       <<  _tropo_mf       << "\" \n"
       << "   grad_mf=\""        <<  _grad_mf       << "\" \n"     
       << "   sigma_C_GPS=\""    <<  _sigma_C_GPS    << "\" \n"
       << "   sigma_L_GPS=\""    <<  _sigma_L_GPS    << "\" \n"
       << "   sigma_C_GLO=\""    <<  _sigma_C_GLO    << "\" \n"
       << "   sigma_L_GLO=\""    <<  _sigma_L_GLO    << "\" \n"     
       << "   sig_init_crd=\""   <<  _sig_init_crd   << "\" \n"
       << "   sig_init_ztd=\""   <<  _sig_init_ztd   << "\" \n"
       << "   minimum_elev=\""   <<  _minimum_elev   << "\" \n"
       << " />\n";
   
  cerr << "\t<!-- process description:\n"
       << "\t phase [0,1]   .. use carrier phase data\n"
       << "\t tropo [0,1]   .. estimate troposphere\n"
       << "\t tropo_model   .. tropospheric model (SAAS, GPT, ...)\n"
       << "\t tropo_mf      .. tropospheric mapping function (COSZ, GMF, ...)\n"
       << "\t grad_mf       .. tropo gradient mapping function (tilt,CHH ...)\n"     
       << "\t sigma_C       .. C/code measurements precision for GPS [m]\n"
       << "\t sigma_L       .. L/phase measurements precision\n"
       << "\t sig_init_crd  .. accuracy of initial coordinates [m]\n"
       << "\t sig_init_ztd  .. accuracy of initial zenith path delay [m]\n"
       << "\t minimum_elev  .. elevation angle cut-off [degree]\n"
       << "\t -->\n\n";

#ifdef BMUTEX  
   lock.unlock();
#endif
   
   _gmutex.unlock(); return;
}

} // namespace
