
#ifndef GSETPROC_H
#define GSETPROC_H

#define XMLKEY_PROC "process"

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements processing setting
  Version: $ Rev: $

  2012-10-23 /JD: created

-*/

#include <string>
#include <iostream>

#ifdef BMUTEX
  #include <boost/thread/mutex.hpp>
#endif

#include "gio/glog.h"
#include "gutils/gtypeconv.h"
#include "gset/gsetbase.h"

using namespace std;
using namespace pugi;

namespace gnut {

class t_gsetproc : public virtual t_gsetbase
{
 public:
   t_gsetproc();
  ~t_gsetproc();
  
   void check();                                 // settings check
   void help();                                  // settings help

   bool   tropo();
   bool   gradient();
   bool   phase();
   bool   single_frq();
   double sigma_C_GPS();
   double sigma_L_GPS();   
   double sigma_C_GLO();
   double sigma_L_GLO();
   double sigma_C_GAL();
   double sigma_L_GAL();
   double sigma_C_BDS();
   double sigma_L_BDS();
   double sigma_C_QZS();
   double sigma_L_QZS();   
   double sig_init_ztd();
   double sig_init_crd();
   double sig_init_amb();
   double sig_init_glo();
   double sig_init_gal();
   double sig_init_bds();
   double sig_init_qzs();   
   double minimum_elev();
   string tropo_mf();
   string grad_mf();   
   string tropo_model();
   
 protected:
   
   bool   _phase;                                // use phase data [true=1, false=0]
   bool   _gradient;                             // estimate troposphere gradinet [true=1, false=0]
   bool   _tropo;                                // estimate troposphere [true=1, false=0]
   bool   _single_frq;                           // use single frequency   
   string _tropo_model;                          // tropo model [SAAS, GPT, VMF1, ... ]
   string _tropo_mf;                             // tropo mapping function [COSZ, NIELL, GMF, VMF1, ... ]
   string _grad_mf;                             // grad mapping function [tilt, CHH]
   double _sigma_C_GPS;                          // C/code measurements precision [m]
   double _sigma_L_GPS;                          // L/phase measurements precision [m]
   double _sigma_C_GLO;                          // C/code measurements precision [m] - GLONASS
   double _sigma_L_GLO;                          // L/phase measurements precision [m] - GLONASS
   double _sigma_C_GAL;                          // C/code measurements precision [m] - Galileo
   double _sigma_L_GAL;                          // L/phase measurements precision [m] - Galileo
   double _sigma_C_BDS;                          // C/code measurements precision [m] - BeiDou
   double _sigma_L_BDS;                          // L/phase measurements precision [m] - BeiDou
   double _sigma_C_QZS;                          // C/code measurements precision [m] - QZSS
   double _sigma_L_QZS;                          // L/phase measurements precision [m] - QZSS
   double _sig_init_ztd;                         // accuracy of initial zenith path delays [m]
   double _sig_init_crd;                         // accuracy of initial coordinates [m]
   double _sig_init_amb;                         // accuracy of initial ambiguity [m]
   double _sig_init_glo;                         // accuracy of initial GLONASS system time difference
   double _sig_init_gal;                         // accuracy of initial Galileo system time difference
   double _sig_init_bds;                         // accuracy of initial BeiDou system time difference
   double _sig_init_qzs;                         // accuracy of initial QZSS system time difference
   double _minimum_elev;                         // elevation angle cut-off [degree]
   
 private:
};

} // namespace

#endif
